import Taro, { request } from '@tarojs/taro';
import store from '../store/index';
import { BASE_SERVER_URL, LogoutCode, SuccessCode } from '@/constants';

export type ResponseDataEntity<T> = {
  code: number;
  msg: string;
  data: T;
  [propName: string]: any;
};

type ResponseEntity = {
  data: ResponseDataEntity<any>;
  [propName: string]: any;
};

const createFetch = (basePath: string) => {
  return (entity: request.Param) => {
    const token = store.getState().userinfo.token;
    return Taro.request({
      ...entity,
      header: token
        ? {
            token: token,
            reqSource: 'wxmini',
          }
        : {
            reqSource: 'wxmini',
          },
      url: basePath + entity.url,
    }).then(({ data }: ResponseEntity) => {
      if (data.code === SuccessCode) {
        return data;
      }
      if (data.code === LogoutCode) {
        Taro.redirectTo({
          url: '/pages/Login/Login',
        });
        throw data;
      }
      Taro.showToast({
        title: data.msg || '网络错误',
        icon: 'none',
      });
      throw data;
    });
  };
};

export const baseFetch = createFetch(BASE_SERVER_URL);

export default createFetch;
