import Taro, { Component, Config } from '@tarojs/taro';
import { ComponentClass } from 'react';
import { View, Text, Button } from '@tarojs/components';
import { getPayOrder } from '../../api/Order/pay';
import { connect } from '@tarojs/redux';
import { Device } from '../../types/Device/Device';
import PrepayConfig from '../../types/Order/Order';
import { Customer } from '../../types/Customer/Customer';
import AES from 'crypto-js/aes';
import Utf8 from 'crypto-js/enc-utf8';
import ECBmode from 'crypto-js/mode-ecb';
import PaddingPkcs7 from 'crypto-js/pad-pkcs7';
// import { paySuccess } from '../../api/Order/paySuccess';
import './pay.scss';
import { str2ab, ab2str } from '@/utils/arrayBuffer';
import { SOCKET_URL } from '@/constants';

enum BlueToothError {
  BlueToothNotOpen = 'BlueToothNotOpen',
  DeviceNotFound = 'DeviceNotFound',
}

type DeviceInfo = {
  deviceId: string;
  uuid: string;
  isPrimary: boolean;
  serviceId: string;
  notifyId: string;
  writeId: string;
  readId: string;
};
type PageStateProps = {
  device: Device;
  prepayConfig: PrepayConfig[];
  userinfo: Customer;
};

type PageDispatchProps = {};

type PageState = {
  payId: undefined | number;
  bluetoothState: boolean;
  deviceInfo: DeviceInfo;
  sockedDone: boolean;
  deviceDone: boolean;
  showerState: boolean;
};

type Iprop = PageStateProps & PageDispatchProps;
interface Pay {
  props: Iprop;
  state: PageState;
}

const StopCode = 1000;
let timer: NodeJS.Timeout | null = null;
let reConnectting: boolean = false;
let socketTask: Taro.SocketTask | null = null;

@connect(({ device, prepayConfig, userinfo }) => ({
  device,
  prepayConfig,
  userinfo,
}))
class Pay extends Component {
  config: Config = {
    navigationBarTitleText: '支付金额',
  };
  constructor(props) {
    super(props);
    this.state = {
      payId: undefined,
      bluetoothState: false,
      sockedDone: false,
      deviceDone: false,
      showerState: false,
      deviceInfo: {
        deviceId: '',
        uuid: '',
        isPrimary: false,
        serviceId: '',
        notifyId: '',
        writeId: '',
        readId: '',
      },
    };
    this.checkBluetoothAndWs();
  }
  componentWillUnmount() {
    // this.closeBluetooth();
    // this.closeDeviceSocket();
  }

  connectDeviceSocket() {
    Taro.connectSocket({ url: SOCKET_URL }).then(task => {
      socketTask = task;
      task.onOpen(() => {
        console.log('onOpen');
        if (reConnectting) {
          this.reConnectDeviceSocket(true);
        }
        this.setState({
          sockedDone: true,
        });
        task.send({ data: str2ab('{}') });
        this.sendDeviceCode();
      });
      task.onMessage(res => {
        const msg: string = ab2str(res.data);
        console.log('获取socket消息: ', msg);
        if (msg === '[0]') {
          console.log('结束蓝牙以及socket: ', msg);
          // this.closeDeviceSocket();
          // this.closeBluetoothConnection();
        } else if (msg === '[]') {
        } else {
          if (msg.length > 100) {
            this.sendMessageToDevice('[]');
          } else if (msg.length > 20) {
            for (let index = 0; index <= Math.floor(msg.length / 20); index++) {
              let str = msg.substring(index * 20, (index + 1) * 20);
              this.sendMessageToDevice(str);
            }
          } else {
            if (msg) {
              this.sendMessageToDevice(msg);
            }
          }
        }
      });
      task.onClose(e => {
        console.log('socked关闭', e, reConnectting, timer);
        this.setState({
          sockedDone: false,
        });
        socketTask = null;
        if (e.code === StopCode) {
          console.log('正确结束socket连接');
        } else {
          console.log('开始重连socket');
          this.reConnectDeviceSocket(false);
        }
      });
    });
  }

  reConnectDeviceSocket(cancel: boolean) {
    if (cancel) {
      timer && clearTimeout(timer);
      timer = null;
      reConnectting = false;
      return;
    }
    console.log(reConnectting, timer);

    if (reConnectting) {
      this.connectDeviceSocket();
    } else if (timer) {
      clearTimeout(timer);
      reConnectting = false;
      timer = null;
      console.log('请保证网络正常');
      Taro.showModal({
        title: '警告',
        content: '请保持网络畅通正常',
      });
    } else {
      timer = setTimeout(() => {
        reConnectting = false;
      }, 10000);
      reConnectting = true;
      console.log(reConnectting, timer);
      this.connectDeviceSocket();
    }
  }

  sendDeviceCode() {
    const { sockedDone, deviceDone } = this.state;
    const { device } = this.props;
    console.log('socket状态：', sockedDone, '蓝牙状态：', deviceDone);

    if (sockedDone && socketTask && deviceDone) {
      let deviceData = '{<' + device.code + '>}';
      console.log('<---发送设备编号:', deviceData);
      socketTask.send({
        data: str2ab(deviceData),
        success: msg => {
          console.log('发送设备编号:', msg, '--->');
        },
        fail: err => {
          console.log('发送设备编号:', err, '--->');
        },
      });
    }
  }

  closeDeviceSocket() {
    console.log('in close', socketTask, timer);
    if (socketTask) {
      socketTask.close({
        code: StopCode,
        complete: () => {
          socketTask = null;
        },
      });
    }
    if (timer) {
      clearTimeout(timer);
    }
  }
  checkIsShower(): boolean {
    const { device } = this.props;
    return device.serviceId === 5;
  }
  checkBluetoothAndWs() {
    if (this.checkIsShower()) {
      console.log('打开蓝牙和WebSocket');
      this.openBluetooth();
      this.connectDeviceSocket();
    }
  }

  openBluetooth() {
    Taro.openBluetoothAdapter()
      .then(res => {
        console.log(res);
        this.setState(
          {
            bluetoothState: true,
          },
          () => {
            this.startConnectDevice();
          },
        );
      })
      .catch(err => {
        console.error(err);
        Taro.onBluetoothAdapterStateChange(res => {
          console.log('adapterState changed, now is', res);
          if (res.available) {
            this.setState({
              bluetoothState: true,
            });
          }
        });
      });
  }

  checkBluetoothState() {
    const { bluetoothState } = this.state;
    return bluetoothState;
  }

  closeBluetoothConnection() {
    const { deviceInfo } = this.state;
    this.setState({
      showerState: false,
    });
    Taro.closeBLEConnection({
      deviceId: deviceInfo.deviceId,
    })
      .then(() => {
        this.setState({
          deviceDone: false,
        });
      })
      .catch(err => {
        console.log('断开蓝牙连接失败！');
        console.error(err);
      });
  }

  closeBluetooth() {
    Taro.closeBluetoothAdapter();
  }

  warnBluetoothOpen() {
    return Taro.showModal({
      title: '警告',
      content: '为保障正常使用，请打开蓝牙功能',
    });
  }

  startDevicesDiscovery() {
    console.log('in startDevicesDiscovery', this.checkBluetoothState());

    if (this.checkBluetoothState()) {
      return Taro.startBluetoothDevicesDiscovery()
        .then(() => {
          const {
            code,
            // isOnlyBluetooth
          } = this.props.device;
          let timer: NodeJS.Timeout | null = null;
          return new Promise((resolve, reject) => {
            // if (isOnlyBluetooth) {
            //   timer = setTimeout(() => {
            //     reject({ msg: BlueToothError.DeviceNotFound });
            //   }, 10000);
            // } else {
            timer = setTimeout(() => {
              reject({ msg: BlueToothError.DeviceNotFound });
            }, 5000);
            // }
            Taro.onBluetoothDeviceFound(res => {
              const devices = res.devices;
              if (devices[0].name === code) {
                let deviceId = devices[0].deviceId;
                Taro.stopBluetoothDevicesDiscovery();
                this.setState({
                  deviceId,
                });
                if (timer) {
                  clearInterval(timer);
                }
                resolve(deviceId);
              }
            });
          });
        })
        .catch(err => {
          // Taro.stopBluetoothDevicesDiscovery();
          if (timer) {
            clearInterval(timer);
          }
          Taro.hideLoading();
          console.error('startBluetoothDevicesDiscovery: ', err);
          // if (err.msg === BlueToothError.DeviceNotFound) {
          Taro.stopBluetoothDevicesDiscovery();
          // this.sendStartShower();
          throw err;
          // } else {
          //   return this.warnBluetoothOpen();
          // }
        });
    } else {
      Taro.hideLoading();
      return this.warnBluetoothOpen().then(res => {
        if (res.confirm) {
          // const { isOnlyBluetooth } = this.props.bluetoothDevice;
          // if (!isOnlyBluetooth) {
          //   Taro.showLoading({
          //     title: '开启中',
          //     mask: true,
          //   });
          //   return new Promise((resolve, reject) => {
          //     setTimeout(() => {
          //       this.sendStartShower();
          //       reject();
          //     }, 5000);
          //   });
          // }
        }
        throw BlueToothError.BlueToothNotOpen;
      });
    }
  }

  createConnection(deviceId: string) {
    console.log(deviceId);
    return new Promise((resolve, reject) => {
      Taro.createBLEConnection({
        deviceId,
      });

      let abStr = '';
      Taro.onBLECharacteristicValueChange(res => {
        console.log(
          `characteristic ${res.characteristicId} has changed, now is ${
            res.value
          }`,
        );
        console.log(res.value);
        let datastr = ab2str(res.value);
        console.log('获取socket消息： ', datastr);
        if (datastr === '<>') {
          console.log('结束蓝牙以及socket: ', datastr);
          // this.closeDeviceSocket();
          this.closeBluetoothConnection();
        } else {
          if (
            datastr.substring(0, 1) === '{' &&
            datastr.substring(datastr.length - 1, datastr.length) === '}'
          ) {
            console.log('<---发送完整数据： ', datastr);
            if (socketTask) {
              socketTask.send({
                data: str2ab(datastr),
                success: msg => {
                  console.log('发送完整数据：', msg, '--->');
                },
                fail: err => {
                  console.log('发送完整数据：', err, '--->');
                },
              });
            }
          } else if (datastr.substring(0, 1) === '{') {
            abStr = datastr;
          } else if (
            abStr.length &&
            datastr.substring(datastr.length - 1, datastr.length) === '}'
          ) {
            abStr += datastr;
            console.log('<---发送完整数据： ', abStr);
            if (socketTask) {
              socketTask.send({
                data: str2ab(abStr),
                success: msg => {
                  console.log('发送完整数据：', msg, '--->');
                },
                fail: err => {
                  console.log('发送完整数据：', err, '--->');
                },
              });
            }
            abStr = '';
          } else {
            console.log('弃掉数据：', datastr);
          }
        }
      });

      Taro.onBLEConnectionStateChange(res => {
        // const { showerState } = this.state;
        console.log(
          `device ${res.deviceId} state has changed, connected: ${
            res.connected
          }`,
        );
        if (res.connected) {
          this.setState({
            deviceDone: true,
            showerState: true,
          });
          resolve(res.deviceId);
        } else {
          console.log(res);
          this.setState({
            deviceDone: false,
          });
          // if (showerState) {
          this.reConnectDeviceBluetooth(); // 只要断了就重连
          // }
          reject('蓝牙未成功连接');
        }
      });
    });
  }

  sendMessageToDevice(msg: string) {
    const { deviceInfo } = this.state;
    console.log(msg, str2ab(msg).byteLength);
    if (deviceInfo.writeId) {
      Taro.writeBLECharacteristicValue({
        deviceId: deviceInfo.deviceId,
        serviceId: deviceInfo.serviceId,
        characteristicId: deviceInfo.writeId,
        value: str2ab(msg),
      });
    }
  }

  reConnectDeviceBluetooth() {
    if (this.checkBluetoothState()) {
      const { deviceInfo } = this.state;
      this.createConnection(deviceInfo.deviceId);
    }
  }

  getDeviceServices(deviceId: string) {
    return Taro.getBLEDeviceServices({
      deviceId: deviceId,
    }).then(services => {
      return { deviceId: deviceId, services: services.services };
    });
  }

  getDeviceCharacter({
    deviceId,
    serviceId,
  }: {
    deviceId: string;
    serviceId: string;
  }) {
    return Taro.getBLEDeviceCharacteristics({
      deviceId,
      serviceId,
    }).then(res => {
      let notifyId = '',
        writeId = '',
        readId = '';
      for (let i = 0; i < res.characteristics.length; i++) {
        let charc = res.characteristics[i];
        if (charc.properties.notify) {
          notifyId = charc.uuid;
        }
        if (charc.properties.write) {
          writeId = charc.uuid;
        }
        if (charc.properties.write) {
          readId = charc.uuid;
        }
      }
      return {
        notifyId,
        writeId,
        readId,
      };
    });
  }

  openNotify({ notifyId, writeId, readId }) {
    this.setState(({ deviceInfo }: PageState) => ({
      deviceInfo: {
        ...deviceInfo,
        notifyId,
        writeId,
        readId,
      },
    }));
    const { deviceInfo } = this.state;
    return Taro.notifyBLECharacteristicValueChange({
      deviceId: deviceInfo.deviceId,
      serviceId: deviceInfo.serviceId,
      characteristicId: notifyId,
      state: true,
    }).then(res => {
      console.log(res);

      let abStr = '';
      Taro.onBLECharacteristicValueChange(res => {
        let msg = ab2str(res.value);
        console.log('获取蓝牙设备消息： ', msg);
        if (msg === '<>') {
          // this.closeBluetoothConnection();
          // this.closeDeviceSocket();
        } else {
          if (
            msg.substring(0, 1) === '{' &&
            msg.substring(msg.length - 1, msg.length) === '}'
          ) {
            console.log('<---发送给socket完整数据： ', msg);
            socketTask &&
              socketTask.send({
                data: str2ab(msg),
                success: msg => {
                  console.log('发送给socket完整数据：', msg, '--->');
                },
                fail: err => {
                  console.log('发送给socket完整数据：', err, '--->');
                },
              });
          } else if (msg.substring(0, 1) === '{') {
            abStr = msg;
          } else if (
            abStr.length &&
            msg.substring(msg.length - 1, msg.length) === '}'
          ) {
            abStr += msg;
            console.log('<---发送给socket完整数据： ', abStr);

            socketTask &&
              socketTask.send({
                data: str2ab(abStr),
                success: msg => {
                  console.log('发送给socket完整数据：', msg, '--->');
                },
                fail: err => {
                  console.log('发送给socket完整数据：', err, '--->');
                },
              });
            abStr = '';
          } else {
            if (abStr.length) {
              abStr += msg;
            } else {
              console.log('弃掉数据：', msg);
            }
          }
        }
      });
    });
  }

  userWarnningHandle(thresholdPrompt, money, beanAmount) {
    Taro.hideLoading();
    console.log('in userWarnningHandle');
    let arr = [money, beanAmount];
    let i = -1;
    return Taro.showModal({
      title: '提示',
      content: thresholdPrompt.replace(/(\{.*?\})/g, () => {
        i++;
        return arr[i];
      }),
    }).then(res => {
      if (res.confirm) {
        Taro.showLoading({
          title: '开启中',
          mask: true,
        });
        return Promise.resolve();
      } else {
        return Promise.reject();
      }
    });
  }

  startConnectDevice() {
    const { sockedDone } = this.state;
    if (!sockedDone) {
      this.connectDeviceSocket();
    }
    this.startDevicesDiscovery()
      .then((deviceId: string) => this.createConnection(deviceId))
      .then(this.getDeviceServices)
      .then(res => {
        const deviceId = res.deviceId;
        const services = res.services;
        let service = services.find(
          item =>
            item.uuid.toLocaleLowerCase() ===
            '6e401103-b5a3-f393-e0a9-e50e24dcca9e',
        );
        if (!service) {
          service = services[0];
        }
        console.log('获取全部services: ', services, '. serviceId: ', service);
        const serviceId = service.uuid;
        this.setState(({ deviceInfo }: PageState) => ({
          deviceInfo: {
            ...deviceInfo,
            deviceId,
            serviceId,
          },
        }));
        return this.getDeviceCharacter({
          deviceId,
          serviceId,
        });
      })
      .then(res => this.openNotify(res))
      .then(() => {
        this.sendDeviceCode();
        // if (!reConnect) {
        //   setTimeout(() => {
        //     this.sendStartShower();
        //   }, 2000);
        // }
      })
      .catch(err => {
        console.error(err);
        Taro.hideLoading();
      });
  }

  sendStartShower() {
    console.log('开启洗浴');
    // const { code } = this.props.device;
    // const { userinfo } = this.props;
    // startShowerEquipment({
    //   deviceCode: code,
    //   customerId: userinfo.customerId.toString(),
    //   customerName: userinfo.customerName,
    //   customerPhone: userinfo.customerPhone,
    // })
    //   .then(res => {
    //     console.log(res);
    //     Taro.showToast({
    //       title: '开启成功！',
    //     });
    //     this.setState({
    //       showerState: true,
    //     });
    //   })
    //   .catch(err => {
    //     console.error(err);
    //     Taro.showToast({
    //       title: err.msg || '开启失败！',
    //       icon: 'none',
    //     });
    //     this.closeBluetoothConnection();
    //   });
  }

  selectPayConfig(id: number) {
    this.setState({
      payId: id,
    });
  }

  payHandle() {
    const {
      device,
      userinfo: { customerId },
    } = this.props;
    const { payId, deviceDone, sockedDone } = this.state;
    if (payId) {
      if (this.checkIsShower()) {
        console.log(deviceDone, sockedDone);
        if (!deviceDone || !sockedDone) {
          return Taro.showModal({
            title: '提示',
            content: '设备连接中...请稍后再试',
          });
        }
      }
      getPayOrder({
        equipmentNum: device.originCode,
        equipmentPos: device.position,
        payType: '2',
        serviceId: device.serviceId,
        smaproCustomerId: customerId,
        smaproPrepayConfigId: payId,
      })
        .then(res => {
          console.log('res: ', res);

          const { payStr } = res;
          let key = customerId.toString();
          while (key.length < 16) {
            key = key + '0';
          }

          const payData = JSON.parse(
            AES.decrypt(payStr, Utf8.parse(key), {
              mode: ECBmode,
              padding: PaddingPkcs7,
            }).toString(Utf8),
          );
          console.log('payData:', payData);
          Taro.requestPayment({
            timeStamp: payData.msg.timeStamp.toString(),
            nonceStr: payData.msg.nonceStr,
            package: payData.msg.package,
            signType: payData.msg.signType,
            paySign: payData.msg.paySign,
          }).then(res => {
            console.log(res);
            // paySuccess({ orderId: outTradeNo });
            // Taro.showLoading({
            //   title: '设备连接中',
            //   mask: true,
            // });
            Taro.showToast({
              title: '支付成功！',
              icon: 'success',
              mask: true,
            });
            this.gobackHandle();
          });
        })
        .catch(err => {
          Taro.showToast({
            title: '发起支付失败!',
            icon: 'none',
          });
          console.error(err);
        });
    } else {
      Taro.showToast({
        title: '请选择预付金额',
        icon: 'none',
      });
    }
  }

  gobackHandle() {
    if (this.checkIsShower()) {
      setTimeout(() => {
        Taro.navigateTo({
          url: '/pages/index/index?', // 需要问号不然回到首页可以取到业务参数
        });
      }, 2000);
      setTimeout(() => {
        this.closeBluetooth();
        this.closeDeviceSocket();
      }, 10000);
    } else {
      setTimeout(() => {
        Taro.navigateTo({
          url: '/pages/index/index?', // 需要问号不然回到首页可以取到业务参数
        });
      }, 2000);
    }
  }

  render() {
    const { device, prepayConfig } = this.props;
    const { payId } = this.state;
    return (
      <View className='Pay'>
        <View className='Pay-info'>
          <View className='Pay-info-item'>
            <Text>服务类型</Text>
            <Text>{device.serviceName ? device.serviceName : ''}</Text>
          </View>
          <View className='Pay-info-item'>
            <Text>设备编号</Text>
            <Text>{device.code ? device.code : ''}</Text>
          </View>
          <View className='Pay-info-item'>
            <Text>设备位置</Text>
            <Text>{device.position}</Text>
          </View>
          <View className='Pay-info-item'>
            <Text>适用费率</Text>
            <View className='Pay-info-rate'>
              {device.rates.map(item => (
                <View key={item.name} className='Pay-info-rate-item'>
                  <Text className='Pay-info-rate-name'>{item.name}</Text>
                  <Text className='Pay-info-rate-mark'>{item.mark}</Text>
                </View>
              ))}
              {/* <View className='Pay-info-rate-item'>
                <Text className='Pay-info-rate-name'>冷水</Text>
                <Text className='Pay-info-rate-mark'>0.02元/100ml</Text>
              </View>
              <View className='Pay-info-rate-item'>
                <Text className='Pay-info-rate-name'>热水</Text>
                <Text className='Pay-info-rate-mark'>0.02元/100ml</Text>
              </View> */}
            </View>
          </View>
        </View>
        <View className='Pay-tip'>
          温馨提示：如本次预付款未全部使用，系统将在使用结束后半小时内自动退还剩余金额，请留意查收！
        </View>
        <View className='Pay-money'>
          {prepayConfig &&
            prepayConfig.length &&
            prepayConfig.map(payConfig => (
              <View
                key={payConfig.id}
                className={`Pay-money-item ${
                  payConfig.id == payId ? 'seleted' : ''
                }`}
                onClick={() => this.selectPayConfig(payConfig.id)}>
                {payConfig.prepayMoney}元
              </View>
            ))}
          {/* <View className='Pay-money-item'>1.00元</View>
          <View className='Pay-money-item'>2.00元</View>
          <View className='Pay-money-item'>5.00元</View>
          <Input className='Pay-money-item' placeholder='输入>0.5数额' /> */}
        </View>
        <Button className='Pay-btn' onClick={this.payHandle}>
          确定支付
        </Button>
      </View>
    );
  }
}

export default Pay as ComponentClass<Iprop, PageState>;
